<?php

namespace Drupal\migrate_wizard\Form;

use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\migrate_wizard\MWConfigBuilder;
use Drupal\migrate_wizard\MWManageDataService;
use Drupal\paragraphs\Entity\Paragraph;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;

/**
 * Configure migrate_wizard.
 */
class OriginBundleFields extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected $mwManageDataService;

  /**
   * {@inheritdoc}
   */
  protected $mwConfigBuilder;

  /**
   * {@inheritdoc}
   */
  protected $fieldsOriginBundle;

  /**
   * {@inheritdoc}
   */
  protected $arrayFieldsOriginBundle;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $entityFieldsDestiny;

  CONST CHECKBOXES = [
    'preserve_nid' => 'Preserve nid',
    'import_users' => 'Import users',
    'import_path_alias' => 'Import path alias',
  ];
  /**
   * {@inheritdoc}
   */
  public function __construct(MWManageDataService $mw_manage_data_service,
                              MWConfigBuilder $mw_config_builder,
                              ConfigFactoryInterface $config_factory,
                              EntityFieldManagerInterface $entity_field_manager,
                              EntityTypeManagerInterface $entity_type_manager) {
    $this->mwManageDataService = $mw_manage_data_service;
    $this->mwConfigBuilder = $mw_config_builder;
    $this->config = $config_factory->getEditable('migrate_wizard.settings');
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('migrate_wizard.migrate_wizard_manage_data_service'),
      $container->get('migrate_wizard.migrate_wizard_config_builder_service'),
      $container->get('config.factory'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'manage_origin_bundle_fields';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['migrate_wizard.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $origin_bundle = NULL) {

    $current_config = $this->config->get($origin_bundle);
    $form['origin_bundle'] = [
      '#type' => 'hidden',
      '#value' => $origin_bundle,
    ];

    //get current url
    $current_url = \Drupal::request()->getRequestUri();
    $current_url = explode('/', $current_url);

    switch ($current_url[3]) {
      case 'list-content-types':
        $form['origin_type'] = [
          '#type' => 'hidden',
          '#value' => 'node',
        ];
        break;
      case 'list-vocabularies':
        $form['origin_type'] = [
          '#type' => 'hidden',
          '#value' => 'vocabulary',
        ];
        break;
      case 'list-users':
        $form['origin_type'] = [
          '#type' => 'hidden',
          '#value' => 'user',
        ];
        $current_config['destiny_type'] = 'user';
        $current_source_type_value = 'user';
        $form['source_type'] = [
          '#type' => 'hidden',
          '#value' => 'user',
        ];
        $default_value_preserve_uid = FALSE;
        if ($current_config && isset($current_config['preserve_uid'])) {
          $default_value_preserve_uid = $current_config['preserve_uid'];
        }
        $form['preserve_uid'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Preserve uid'),
          '#default_value' => $default_value_preserve_uid,
        ];
        $default_value_import_cas = FALSE;
        if ($current_config && isset($current_config['import_cas'])) {
          $default_value_import_cas = $current_config['import_cas'];
        }
        $form['import_cas'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Import cas'),
          '#default_value' => $default_value_import_cas,
        ];
        break;
    }

    $destiny_type = 'node';
    if ($current_config && isset($current_config['destiny_type'])) {
      $destiny_type = $current_config['destiny_type'];
    }
    $form['destiny_type'] = [
      '#type' => 'hidden',
      '#value' => $destiny_type,
      '#prefix' => '<div class="destiny_type_box">',
      '#suffix' => '</div>',
    ];

    $default_value = FALSE;
    if ($current_config && isset($current_config['build_migration_config'])) {
      $default_value = $current_config['build_migration_config'];
    }
    $form['build_migration_config'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Build migration config'),
      '#default_value' => $default_value,
    ];

    if($current_config == NULL || $current_config['destiny_type'] != 'user'){
      foreach(self::CHECKBOXES as $key => $label) {

        if ($current_config && isset($current_config[$key])) {
          $default_value = $current_config[$key];
        }
        $form[$key] = [
          '#type' => 'checkbox',
          '#title' => $this->t($label),
          '#default_value' => $default_value,
        ];
      }
      $form['languages']['default_lang'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Default translation') . ' (' . $this->mwManageDataService->getOriginDefaultLangcode() . ')',
        '#default_value' => TRUE,
        '#disabled' => TRUE,
        '#checked' => TRUE,
      ];
      $origin_languages = $this->mwManageDataService->getOriginLanguages($form["origin_type"]["#value"], $origin_bundle);
      foreach($origin_languages as $language){
        $default_value = FALSE;
        if ($current_config != null && isset(reset($current_config)["languages"][$language])) {
          $default_value = TRUE;
        }
        $form['languages'][$language.'_lang'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Translation') . ' (' . $language . ')',
          '#default_value' => $default_value,
        ];
      }
    }

    $options_source_origin_type_node = $this->getOriginNodeTypes();
    $options_source_origin_type_vocabulary = $this->getOriginVocavularyTypes();
    $both_options_source = ['none' => 'Select dependency'] + ['Content types' => $options_source_origin_type_node, 'Vocabularies' => $options_source_origin_type_vocabulary];

    if($current_config == NULL || $current_config['destiny_type'] != 'user'){

      $options_source_type = $this->mwManageDataService->getCurrentContentTypes();
      $options_source_type_vocabulary = $this->mwManageDataService->getCurrentVocabularies();

      $form['source_type'] = [
        '#type' => 'select',
        '#title' => $this->t('Content types'),
        '#options' => ['none' => 'Select content type'] + $options_source_type,
        '#ajax' => [
          'callback' => [$this, 'getFieldsDestinyNode'],
          'event' => 'change',
          'wrapper' => 'source-destiny-fields-wrapper',
        ],
        '#value' => $current_source_type_value = ($current_config == NULL  ? 'none' : array_key_first($current_config)),
        '#prefix' => '<div class="source_type_box">',
        '#suffix' => '</div>',
      ];

      $form['source_type_vocabulary'] = [
        '#type' => 'select',
        '#title' => $this->t('Vocabularies'),
        '#options' => ['none' => 'Select vocabulary'] + $options_source_type_vocabulary,
        '#ajax' => [
          'callback' => [$this, 'getFieldsDestinyVocabulary'],
          'event' => 'change',
        ],
        '#value' => $current_source_type_vocabulary_value = ($current_config == NULL  ? 'none' : array_key_first($current_config)),
        '#prefix' => '<div class="source_type_vocabulary_box">',
        '#suffix' => '</div>',
      ];

      $current_source_type_date_field = '';
      if ($current_config && isset($current_config['source_type_date_field'])) {
        $current_source_type_date_field = $current_config['source_type_date_field'];
      }
      $form['source_type_date_field'] = [
        '#type' => 'date',
        '#title' => $this->t('Import from date'),
        '#date_date_format' => 'd/m/Y',
        '#default_value' => date('d/m/Y'),
        '#value' => $current_source_type_date_field,
      ];
    }

    $form['source_destiny_fields'] = [
      '#type' => 'fieldset',
      '#title' => t('Fields'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#prefix' => '<div class="source-destiny-fields-wrapper" id="source-destiny-fields-wrapper">',
      '#suffix' => '</div>',
    ];

    switch($current_url[3]){
      case 'list-content-types':
        $this->fieldsOriginBundle[] = (object) [
          'field_name' => 'title',
          'type' => 'text',
          'data' => serialize(['label' => 'Title']),
          'fc_data' => serialize([]),
        ];
        $this->fieldsOriginBundle = array_merge($this->fieldsOriginBundle, $this->mwManageDataService->getFieldsOriginNode($origin_bundle));
        break;
      case 'list-vocabularies':
        $this->fieldsOriginBundle[] = (object) [
          'field_name' => 'name',
          'type' => 'text',
          'data' => serialize(['label' => 'name']),
          'fc_data' => serialize([]),
        ];
        $this->fieldsOriginBundle = array_merge($this->fieldsOriginBundle, $this->mwManageDataService->getFieldsOriginVocabulary($origin_bundle));
        break;
      case 'list-users':
        $this->fieldsOriginBundle = $this->mwManageDataService->getFieldsOriginNode($origin_bundle);
        break;
    }

    $current_config_node = [];
    if (is_array($current_config)) {
      $current_config_node = reset($current_config);
    }
    $generic_current_source_type_value = $current_source_type_value != 'none' ? $current_source_type_value : $current_source_type_vocabulary_value;
    $current_options_source_destiny_fields_array['fields_array'] = ['none' => 'Select a field'];
    $current_options_source_destiny_fields_array['fields_array_destiny_type'] = [];
    if($current_config != NULL){
      $current_options_source_destiny_fields_array = $this->getDestinyFieldsByBundle($destiny_type, $generic_current_source_type_value);
    }
    $form['source_destiny_fields']['workbench_access_node'] = [
      '#type' => 'select',
      '#title' => 'workbench access node - workbench_access_node (workbench_access_node)',
      '#options' => $current_options_source_destiny_fields_array['fields_array'],
      '#validated' => TRUE,
      '#value' => $current_config != NULL && isset($current_config_node['workbench_access_node']['destiny']) ? $current_config_node['workbench_access_node']['destiny'] : 'none',
    ];

    $form['source_destiny_fields']['workbench_access_node/dependency/workbench_access_node'] = [
      '#type' => 'select',
      '#title' => 'select dependency workbench access node',
      '#options' => $both_options_source,
      '#validated' => TRUE,
      '#attributes' => ['style' => 'margin-left: 50px;'],
      '#value' => $current_config != NULL && isset($current_config_node['workbench_access_node']['dependencies']['workbench_access_node']) ? $current_config_node['workbench_access_node']['dependencies']['workbench_access_node'] : 'none',
    ];

    $form['source_destiny_fields_type'] = [
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#attributes' => ['style' => 'display: none;'],
      '#prefix' => '<div class="source-destiny-fields_type-wrapper" id="source-destiny-fields-type_wrapper">',
      '#suffix' => '</div>',
    ];

    foreach ($current_options_source_destiny_fields_array['fields_array_destiny_type'] as $key => $current_options_source_destiny_field_type) {
      $form['source_destiny_fields_type'][$key . '_type'] = [
        '#type' => 'hidden',
        '#name' => $key . '_type',
        '#value' => $current_options_source_destiny_field_type,
      ];
    }

     $this->buildFieldsForm($form, $current_options_source_destiny_fields_array['fields_array'], $generic_current_source_type_value, $both_options_source, $current_config, $current_config_node, $this->fieldsOriginBundle);

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];
    $form['#attached']['library'][] = 'migrate_wizard/form';
    return $form;
  }

  private function buildFieldsForm(&$form, $current_options_source_destiny_fields, $generic_current_source_type_value, $both_options_source, $current_config, $current_config_node, $fields, $depth = 0, $field_parent = '', $parent_prefix = ''){
    foreach ($fields as $field) {

      $current_data = unserialize($field->data);
      $current_fc_data = unserialize($field->fc_data);

      $depth_indent = $depth*50;
      $depth_indent_extra = $depth_indent+50;
      $prefix = '';
      $type_field = $field->type;
      if ($field->type == 'datetime' && count($current_fc_data["indexes"]) > 1) {
        $prefix = 'daterange/';
        $type_field = 'daterange';
      }
      $current_value = $current_config != NULL && isset($current_config_node[$field->field_name]) ? $current_config_node[$field->field_name]['destiny'] : 'none';
      if($depth > 0){
        $prefix = $parent_prefix.$field_parent . "/parent$depth/".$type_field.'/';
        if(isset($current_config_node[$field_parent]['childs']) && isset($current_config_node[$field_parent]['childs'][$field->field_name])){
          $current_config_node[$field->field_name] = $current_config_node[$field_parent]['childs'][$field->field_name];
        }
        $current_value = $current_config != NULL && isset($current_config_node[$field->field_name]) ? $current_config_node[$field->field_name]['destiny'].'/'.$current_config_node[$field->field_name]['bundle'] : 'none';
        $destiny_explode = explode('/', $current_config_node[$field->field_name]['destiny']);
        if(count($destiny_explode) > 1){
          $current_value = $current_config_node[$field->field_name]['destiny'];
        }
      }
      $form['source_destiny_fields'][$prefix . $field->field_name] = [
        '#type' => 'select',
        '#title' => $current_data['label'] . ' - ' . $field->field_name . ' (' . $type_field . ')',
        '#options' => $current_options_source_destiny_fields,
        '#validated' => TRUE,
        '#attributes' => ['style' => 'margin-left: '.$depth_indent.'px;'],
        '#label_attributes' => ['style' => 'margin-left: '.$depth_indent.'px;'],
        '#value' => $current_value,
      ];

      //if type field is datetime add format to transform
      if ($type_field == 'datetime') {

        $form['source_destiny_fields'][$prefix . $field->field_name]['#prefix'] = '<div class="datetime_mw">';
        $form['source_destiny_fields'][$prefix . $field->field_name]['#suffix'] = '</div>';

        $form['source_destiny_fields'][$prefix . $field->field_name . '/format'] = [
          '#type' => 'textfield',
          '#title' => t('Format Y-m-d H:i:s'),
          '#validated' => TRUE,
          '#value' => $current_config != NULL && isset($current_config_node[$field->field_name]['format']) ? $current_config_node[$field->field_name]['format'] : '',
          '#attributes' => ['style' => 'margin-left: '.$depth_indent.'px;'],
          '#label_attributes' => ['style' => 'margin-left: '.$depth_indent.'px;'],
          '#prefix' => '<div class="format_date_mw">',
          '#suffix' => '</div>',
        ];
      }

      if ($field->type == 'file' || $field->type == 'image') {
        $currentValue = $current_config != NULL && isset($current_config[$generic_current_source_type_value][$field->field_name]['media_field']) ? $current_config[$generic_current_source_type_value][$field->field_name]['media_field'] : [];
        $fields_media = ['none' => 'Select field media'] + $this->mwManageDataService->getFieldsMedia();
        $form['source_destiny_fields']['listmedias/' . $field->field_name] = [
          '#type' => 'select',
          '#title' => t('Select field media'),
          '#options' => $fields_media,
          '#validated' => TRUE,
          '#attributes' => ['style' => 'margin-left: '.$depth_indent_extra.'px;'],
          '#label_attributes' => ['style' => 'margin-left: '.$depth_indent_extra.'px;'],
          '#value' => $currentValue,
        ];
      }

      if ($field->type == 'entityreference' || $field->type == 'taxonomy_term_reference' || $field->type == 'node_reference') {
        $target_bundles = [];
        if ($field->type == 'entityreference') {
          $target_bundles = $current_fc_data["settings"]["handler_settings"]['target_bundles'];
        }
        if ($field->type == 'taxonomy_term_reference') {
          $target_bundles = [$current_fc_data["settings"]["allowed_values"][0]["vocabulary"] => $current_fc_data["settings"]["allowed_values"][0]["vocabulary"]];
        }
        if ($field->type == 'node_reference') {
          foreach($current_fc_data["settings"]["referenceable_types"] as $key => $value) {
            if(is_string($value)){
              $target_bundles[$key] = $value;
            }
          }
        }

        foreach ($target_bundles as $key => $value) {
          $current_value = [];
          $reset_current_config = [];
          if ($current_config != NULL) {
            $reset_current_config = reset($current_config);
          }
          if ($reset_current_config != NULL && isset($reset_current_config[$field->field_name]['dependencies'][$key])) {
            $current_value = $reset_current_config[$field->field_name]['dependencies'][$key];
          }

          $form['source_destiny_fields'][$field->field_name . '/dependency/' . $key] = [
            '#type' => 'select',
            '#title' => 'Select dependency ' . $key,
            '#options' => $both_options_source,
            '#validated' => TRUE,
            '#attributes' => ['style' => 'margin-left: '.$depth_indent_extra.'px;'],
            '#label_attributes' => ['style' => 'margin-left: '.$depth_indent_extra.'px;'],
            '#value' => $current_value,
          ];
        }
      }
      if ($type_field == 'field_collection') {
        $this->getFieldsCollection($field, $form, $current_options_source_destiny_fields, $generic_current_source_type_value, $both_options_source, $current_config, $current_config_node, $depth + 1, $prefix);
      }
      $this->arrayFieldsOriginBundle[$field->field_name] = $field->type;
    }
  }

  private function getFieldsCollection($field, &$form, $current_options_source_destiny_fields, $generic_current_source_type_value, $both_options_source, $current_config, $current_config_node, $depth, $prefix){
    $fieldsCollection = $this->mwManageDataService->getFieldsCollectionByBundle($field->field_name);
    $this->buildFieldsForm($form, $current_options_source_destiny_fields, $generic_current_source_type_value, $both_options_source, $current_config, $current_config_node, $fieldsCollection, $depth, $field->field_name, $prefix);
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsDestinyNode(array &$form, FormStateInterface $form_state) {
    $user_input = $form_state->getUserInput();
    $bundle = $user_input['source_type'];
    $fields_array = $this->getDestinyFieldsByBundle('node', $bundle);
    $current_options_source_destiny_fields_type = $fields_array['fields_array_destiny_type'];

    $form['source_destiny_fields_type'] = [
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
      '#attributes' => ['style' => 'display: none;'],
      '#prefix' => '<div class="source-destiny-fields_type-wrapper" id="source-destiny-fields-type_wrapper">',
      '#suffix' => '</div>',
    ];
    foreach ($current_options_source_destiny_fields_type as $key => $current_options_source_destiny_field_type) {
      $form['source_destiny_fields_type'][$key . '_type'] = [
        '#type' => 'hidden',
        '#name' => $key . '_type',
        '#value' => $current_options_source_destiny_field_type,
      ];
    }

    foreach ($this->fieldsOriginBundle as $field) {
      $form['source_destiny_fields'][$field->field_name]['#options'] = $fields_array['fields_array'];
    }
    $form['source_destiny_fields']['workbench_access_node']['#options'] = $fields_array['fields_array'];
    $form['source_type_vocabulary']['#value'] = 'none';
    $form['destiny_type']['#value'] = 'node';
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand(".source-destiny-fields-wrapper", $form['source_destiny_fields']));
    $response->addCommand(new ReplaceCommand(".source-destiny-fields_type-wrapper", $form['source_destiny_fields_type']));
    $response->addCommand(new ReplaceCommand(".source_type_vocabulary_box", $form['source_type_vocabulary']));
    $response->addCommand(new ReplaceCommand(".destiny_type_box", $form['destiny_type']));
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsDestinyVocabulary(array &$form, FormStateInterface $form_state) {
    $user_input = $form_state->getUserInput();
    $bundle = $user_input['source_type_vocabulary'];
    $fields_array = $this->getDestinyFieldsByBundle('taxonomy_term', $bundle);
    foreach ($this->fieldsOriginBundle as $field) {
      $form['source_destiny_fields'][$field->field_name]['#options'] = $fields_array['fields_array'];
    }

    $form['source_type']['#value'] = 'none';
    $form['destiny_type']['#value'] = 'taxonomy_term';
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand(".source-destiny-fields-wrapper", $form['source_destiny_fields']));
    $response->addCommand(new ReplaceCommand(".source_type_box", $form['source_type']));
    $response->addCommand(new ReplaceCommand(".destiny_type_box", $form['destiny_type']));
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  protected function getDestinyFieldsByBundle($type, $bundle) {

    if ($bundle != 'none') {
      $this->entityFieldsDestiny = $this->entityFieldManager->getFieldDefinitions($type, $bundle);
    }
    if($bundle == 'user'){
      unset($this->entityFieldsDestiny['name']);
    }
    //check if field is paragraph
    $this->checkIfFieldIsParagraph($this->entityFieldsDestiny);
    $fields_array = [];
    $fields_array_destiny_type = [];
    foreach ($this->entityFieldsDestiny as $field_name => $field) {
      $label = $field->getLabel();
      if ($label instanceof TranslatableMarkup) {
        $label = $label->getUntranslatedString();
      }
      $field_name_explode = explode('/', $field_name);
      if(count($field_name_explode) > 1){
        $label = $label.' ('.$field_name_explode[1].')';
      }
      $fields_array[$field_name] = $label;
      $fields_array_destiny_type[$field->getName()] = $field->getType();
    }
    ksort($fields_array);
    $fields_array = ['none' => 'Select a field'] + $fields_array;

    $array_return = [
      'fields_array' => $fields_array,
      'fields_array_destiny_type' => $fields_array_destiny_type,
    ];
    return $array_return;
  }

  private function checkIfFieldIsParagraph($fields){
    $base_fields = $this->entityFieldManager->getFieldDefinitions('paragraph', null);
    foreach ($fields as $name_field => $field) {
      if ($field->getSetting('target_type') == 'paragraph') {
        $bundle = $field->getTargetBundle();
        $handler_settings = $field->getSetting('handler_settings');
        foreach($handler_settings['target_bundles'] as $target_bundles){
          unset($this->entityFieldsDestiny[$name_field]);
          $this->entityFieldsDestiny[$name_field.'/'.$bundle] = $field;
          $full_fields = $this->entityFieldManager->getFieldDefinitions('paragraph', $target_bundles);
          $extra_fields = array_diff_key($full_fields, $base_fields);
          $this->checkIfFieldIsParagraph($extra_fields);
          foreach ($extra_fields as $paragraph_bundle => $paragraph_field) {
            unset($this->entityFieldsDestiny[$paragraph_field->getName()]);
            $bundle_paragraph_field = $paragraph_field->getTargetBundle();
            $this->entityFieldsDestiny[$paragraph_field->getName().'/'.$bundle_paragraph_field] = $paragraph_field;
          }
        }
      }
    }
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $user_inputs = $form_state->getUserInput();
    if (isset($user_inputs['source_type']) && ($user_inputs['source_type'] != 'none') || (isset($user_inputs['source_type_vocabulary']) && $user_inputs['source_type_vocabulary'] != 'none')) {
      $array_config = [];
      foreach ($this->fieldsOriginBundle as $field) {
        if (isset($user_inputs[$field->field_name])) {
          $destiny = $user_inputs[$field->field_name];
          $destiny_explode = explode('/', $destiny);
          if(count($destiny_explode) > 1){
            $bundle = $destiny_explode[1];
          }
          $target_bundles = [];
          if($user_inputs[$field->field_name] != 'none' && $this->entityFieldsDestiny[$user_inputs[$field->field_name]] != NULL && $this->entityFieldsDestiny[$user_inputs[$field->field_name]]->getFieldStorageDefinition()->getSettings() == ['target_type' => 'paragraph']){
            foreach($this->entityFieldsDestiny[$user_inputs[$field->field_name]]->getSettings()['handler_settings']['target_bundles'] as $target_bundle){
              $target_bundles = $target_bundle;
            }
          }
          $array_config[$field->field_name] = [
            'type' => $this->arrayFieldsOriginBundle[$field->field_name],
            'destiny' => $user_inputs[$field->field_name],
            'destiny_type' => $user_inputs[$user_inputs[$field->field_name] . '_type'] ?? '',
            'bundle' => $bundle ?? 'none',
            'target_bundles' => $target_bundles
          ];
        }
      }
      $array_config['languages']['default'] = $this->mwManageDataService->getOriginDefaultLangcode();
      $array_destiny_media_field = [];
      foreach ($user_inputs as $key => $value) {
        if($value == 'none'){
          continue;
        }
        if (strpos($key, 'dependency') !== FALSE && $key != 'workbench_access_node/dependency/workbench_access_node') {
          $key_sources = explode('/', $key);
          $array_config[$key_sources[0]]['dependencies'][$key_sources[2]] = $value;
          $array_config[$key_sources[0]]['migration_lookup'][] = $user_inputs[$key];
        }
        if (strpos($key, 'listmedias') !== FALSE) {
          $key_sources = explode('/', $key);
          $array_config[$key_sources[1]]['media_field'] = $value;

          // Check if have more than one to same destiny.
          if (!array_key_exists($user_inputs[$key_sources[1]], $array_destiny_media_field) ||
            (array_key_exists($user_inputs[$key_sources[1]], $array_destiny_media_field) && $array_destiny_media_field[$user_inputs[$key_sources[1]]] != [$key_sources[1] => $value])) {
            $array_destiny_media_field[$user_inputs[$key_sources[1]]][] = [$key_sources[1] => $value];
          }
        }
        if (strpos($key, 'daterange') !== FALSE) {
          $key_sources = explode('/', $key);
          $array_config[$key_sources[1]]['type'] = 'daterange';
          $array_config[$key_sources[1]]['destiny'] = $value;
        }
        if (strpos($key, '/format') !== FALSE) {
          $key_sources = explode('/', $key);
          $array_config[$key_sources[0]]['format'] = $value;
        }
        if (strpos($key, '/parent') !== FALSE) {
          $key_sources = explode('/', $key);
          $this->setParentConfig($array_config[$key_sources[0]], $user_inputs, $key_sources, $value);
        }
        if (strpos($key, '_lang') !== FALSE && $value == 1) {
          $langcode = str_replace('_lang', '', $key);
          $array_config['languages'][$langcode] = $langcode;
        }
      }

      foreach ($array_destiny_media_field as $key => $value) {
        if (count($value) > 1) {
          foreach ($value as $field_destiny) {
            $array_config[key($field_destiny)]['files_mwcustom'] = [$key => reset($field_destiny)];
          }
        }
      }

      if ($user_inputs["workbench_access_node"] != 'none') {
        $array_config['workbench_access_node']['destiny'] = $user_inputs["workbench_access_node"];
        $array_config['workbench_access_node']['type'] = 'workbench_access_node';
        $array_config['workbench_access_node']['dependencies']['workbench_access_node'] = $user_inputs["workbench_access_node/dependency/workbench_access_node"];
        $array_config['workbench_access_node']['migration_lookup'] = $user_inputs["workbench_access_node/dependency/workbench_access_node"];
      }
      $source = '';
      switch($user_inputs["destiny_type"]){
        case 'node':
        case 'user':
          $source = 'source_type';
          break;
        case 'taxonomy_term':
          $source = 'source_type_vocabulary';
          break;
      }
      $source_key = $user_inputs[$source];
      $array_config = [
        $source_key => $array_config,
      ];

      $array_config['build_migration_config'] = $user_inputs["build_migration_config"];

      if($user_inputs["destiny_type"] != 'user'){
        $array_config['preserve_nid'] = $user_inputs["preserve_nid"];
        $array_config['import_users'] = $user_inputs["import_users"];
        $array_config['import_path_alias'] = $user_inputs["import_path_alias"];
        $array_config['source_type_date_field'] = $user_inputs["source_type_date_field"];
      }else{
        $array_config['preserve_uid'] = $user_inputs["preserve_uid"];
        $array_config['import_cas'] = $user_inputs["import_cas"];
      }

      $array_config['destiny_type'] = $user_inputs["destiny_type"];
      $array_config['origin_type'] = $user_inputs["origin_type"];

      $this->config->set($user_inputs['origin_bundle'], $array_config)->save();
      if ($user_inputs["build_migration_config"] && $user_inputs["destiny_type"] != 'user') {
        $this->mwConfigBuilder->buildMigrationConfig($user_inputs['origin_bundle'], $array_config);
      }else{
        $this->mwConfigBuilder->buildMigrationUserConfig($user_inputs, $array_config);
      }
    }
  }

  private function setParentConfig(&$array_config_field, $user_inputs, $key_sources, $value){
    foreach($key_sources as $key_key_sources => $key_source){
      if (strpos($key_source, 'parent') !== FALSE && $key_key_sources > 1) {
        $current_key_sources = $key_sources;
        unset($current_key_sources[$key_key_sources-4]);
        unset($current_key_sources[$key_key_sources-3]);
        unset($current_key_sources[$key_key_sources-2]);
        $this->setParentConfig($array_config_field['childs'][$key_sources[3]], $user_inputs, array_values($current_key_sources), $value);
      }
    }
    if(!isset($array_config_field['childs'][$key_sources[3]])){
      $value_explode = explode('/', $value);
      $destiny = $value_explode[0];
      $target_bundles = [];
      if($this->entityFieldsDestiny[$value]->getFieldStorageDefinition()->getSettings() == ['target_type' => 'paragraph']){
        $destiny = $value;
        foreach($this->entityFieldsDestiny[$value]->getSettings()['handler_settings']['target_bundles'] as $target_bundle){
          $target_bundles = $target_bundle;
        }
      }
      $array_config_field['childs'][$key_sources[3]] = [
        'type' => $key_sources[2],
        'destiny' => $destiny,
        'destiny_type' => $user_inputs[$value_explode[0] . '_type'],
        'bundle' => $value_explode[1],
        'target_bundles' => $target_bundles,
      ];
    }
  }

  private function getOriginNodeTypes(){
    $options_source_origin_type_node = [];
    foreach($this->entityTypeManager->getStorage('origin_node')->loadMultiple() as $key => $value){
      $options_source_origin_type_node[$key] = $value->label();
    }
    return $options_source_origin_type_node;
  }

  private function getOriginVocavularyTypes(){
    $options_source_origin_type_vocabulary = [];
    foreach($this->entityTypeManager->getStorage('origin_vocabulary')->loadMultiple() as $key => $value){
      $options_source_origin_type_vocabulary[$key] = $value->label();
    }
    return $options_source_origin_type_vocabulary;
  }
}
